.octostar-button {
    display: inline-flex;
    align-items: center;
    justify-content: center;
    font-weight: 400;
    padding: 0.25rem 0.75rem;
    border-radius: 0.5rem;
    min-height: 38.4px;
    margin: 0;
    line-height: 1.6;
    color: inherit;
    width: auto;
    user-select: none;
    background-color: rgb(255, 255, 255);
    border: 1px solid rgba(9, 30, 66, 0.2);
  
    &:hover {
      border-color: rgb(255, 75, 75);
      color: rgb(255, 75, 75);
    }
  }
  