.octostar-button {
  display: inline-flex;
  align-items: center;
  justify-content: center;
  font-weight: 400;
  padding: 0.25rem 0.75rem;
  border-radius: 0.5rem;
  min-height: 38.4px;
  margin: 0;
  line-height: 1.6;
  color: inherit;
  width: auto;
  user-select: none;
  background-color: #ffffff;
  border: 1px solid rgba(9, 30, 66, 0.2);
}
.octostar-button:hover {
  border-color: #ff4b4b;
  color: #ff4b4b;
}
.antd-uploader {
  border: 1px dashed #d9d9d9;
  border-radius: 4px;
  text-align: center;
  background-color: #fafafa;
  cursor: pointer;
  transition: border-color 0.3s ease;
}
.antd-uploader:hover {
  border-color: #40a9ff;
}
.upload-icon {
  color: #40a9ff;
  font-size: 30px;
  margin-bottom: 8px;
}
.upload-text {
  display: block;
  color: rgba(0, 0, 0, 0.45);
  font-size: 18px;
  margin-bottom: 16px;
}
body {
  padding: 0;
  margin: 0;
}
.context-menu {
  cursor: pointer;
  width: 100%;
  padding: 6px;
  height: 30px;
}
.context-menu:hover {
  background-color: #f5f5f5;
}
