from typing import List, Union

from octostar_streamlit.core.entities import EdgeSpec, Entity
from octostar_streamlit.core.params_base_model import ParamsBaseModel

# TODO: Typing for subscribeToChanges params